<?php
namespace BotGuardLoadBalancer;

use TrueBV\Punycode;

abstract class BaseISPFunction {

	protected $theme;
	protected $lang;
	protected $settings;
	protected $config_dir;

	public function __construct() {
		$this->theme = 'orion';
		$this->lang = 'en';
		if (isset($_SERVER['HTTP_COOKIE'])) {
			parse_str(strtr($_SERVER['HTTP_COOKIE'], array('&' => '%26', '+' => '%2B', ';' => '&')), $_COOKIE);
		}
		if (isset($_COOKIE['ispmgrlang5'])) {
			list($this->theme, $this->lang, $session) = explode(':', $_COOKIE['ispmgrlang5']);
		}

		$this->config_dir = '/usr/local/mgr5/addon';
		$this->settings = $this->getSettings($this->config_dir . '/botguard_load_balancer.json');
		if (!isset($this->settings['options'])) {
			$this->settings['options'] = [];
		}
	}

	abstract public function execute(string $func, string $action = '');

	public function error(string $msg, string $func, string $action = '') {
		echo <<<XML
<?xml version="1.0" encoding="UTF-8"?>
<doc lang="{$this->lang}" themename="{$this->theme}" func="{$func}" action="{$action}">
	<banner id="{$msg}" status="2" />
</doc>
XML;
	}

	protected function getDomains() {
		$domains = [];

		$xml = @shell_exec("/usr/local/mgr5/sbin/mgrctl -m {$_SERVER['MGR_NAME']} webdomain --format xml 2>/dev/null");
		if (!$xml) {
			return $domains;
		}

		$all_domains = simplexml_load_string($xml);
		if (property_exists($all_domains, 'elem')) {
			if ($all_domains->elem->count() > 1) {
				foreach ($all_domains->elem as $domain) {
					if ($domain->owner == $_SERVER['AUTH_USER'] || intval($_SERVER['AUTH_LEVEL']) >= 29) {
						$domains[$this->punycode((string)$domain->name)] = $domain;
					}
				}
			} else {
				if ($all_domains->elem->owner == $_SERVER['AUTH_USER'] || intval($_SERVER['AUTH_LEVEL']) >= 29) {
					$domains[$this->punycode((string)$all_domains->elem->name)] = $all_domains->elem;
				}
			}
		}

		return $domains;
	}

	protected function punycode(string $domain) {
		$p = new Punycode();
		return $p->encode($domain);
	}

	private function getSettings(string $config_json) {
		try {
			$json = @file_get_contents($config_json);
			if (!$json) {
				throw new \Exception($config_json . ' was not found');
			}
			$rslt = @json_decode($json, true);
			if (!$rslt) {
				throw new \Exception($config_json . ' is corrupted');
			}
			return $rslt;
		} catch (\Exception $ex) {
			return [
				'botguard-api-key' => '',
				'botguard-primary-server' => '',
				'botguard-secondary-server' => '',
				'options' => []
			];
		}
	}

}
